"""Spark related utils."""
#!/usr/bin/env python

import pyspark


CURRENT_CONTEXT = None

def get_context(app_name='SparkJob'):
    """Get new Spark context."""
    global CURRENT_CONTEXT
    if CURRENT_CONTEXT is None:
        conf = pyspark.SparkConf().setAppName(app_name)
        CURRENT_CONTEXT = pyspark.SparkContext(conf=conf)
    return CURRENT_CONTEXT



#!/usr/bin/env python
"""The missing PySpark utils steup file."""

import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pyspark_utils",
    version="1.3.0",
    license="Apache License Version 2.0",
    author="Xiangquan Xiao",
    author_email="xiaoxiangquan@gmail.com",
    description="The missing PySpark utils",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/xiaoxq/pyspark-utils",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "pyspark",
    ],
)



"""The missing PySpark operators."""
#!/usr/bin/env python

import operator


def do_key(func):
    """Return a function which transforms the key."""
    return (lambda kv: (func(kv[0]), kv[1]))

def do_tuple(func):
    """Return a function which transforms a tuple."""
    return (lambda a_tuple: func(*a_tuple))

def do_elems(func):
    """Return a function which transforms all elements of the input."""
    return (lambda elems:
        (func(elem) for elem in elems) if isinstance(elems, tuple) else
        [func(elem) for elem in elems])

def filter_key(func):
    """Return a function which filters the key."""
    return (lambda kv: func(kv[0]))

def filter_value(func):
    """Return a function which filters the value."""
    return (lambda kv: func(kv[1]))

def filter_path(patterns):
    """Return a function which filters paths by fnmatch style patterns."""
    return (lambda path: next((True for pat in patterns if fnmatch.fnmatch(path, pat)), False))

def identity(elem):
    """Return the argument itself."""
    return elem

def substract_keys(pair_rdd, key_rdd):
    """Remove entries of pair_rdd whose key is in the key_rdd."""
    return pair_rdd.subtractByKey(key_rdd.keyBy(identity))

def filter_keys(pair_rdd, whitelist_key_rdd, blacklist_key_rdd=None):
    """
    Only keep entries of pair_rdd whose key is in the whitelist and not in the blacklist.
    The given keys rdd should be distinct.
    """
    pair_rdd = pair_rdd.join(whitelist_key_rdd.keyBy(identity)).mapValues(operator.itemgetter(0))
    if blacklist_key_rdd is not None:
        pair_rdd = pair_rdd.subtractByKey(blacklist_key_rdd.keyBy(identity))
    return pair_rdd

def swap_kv(kv):
    """Swap a key-value tuple."""
    return (kv[1], kv[0])
